define([ 'utils' ], function(Utils) {
	var xhrTimeout;
	var timeout = 30000;
	var TIMEOUT_MSG = '连接超时，请稍后重试';
	var core = {
		/* eslint-disable */
		// Method that performs the ajax request
		ajax : function(method, url, args, showindicator) { 
			// Creating a promise
			var promise = new Promise(function(resolve, reject) {
				// Instantiates the XMLHttpRequest
				var xhr = new XMLHttpRequest();
				var uri = SERVICE_DOMAIN_URL + url; 
				// var uri = url;
				if (showindicator||showindicator == undefined) {  
					App.showIndicator();
				}
				xhr.open(method, uri);
				if (args && (method === 'POST' || method === 'PUT')) {
					xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded;charset=UTF-8');
					xhr.send(Utils.serialize(args));
				} else {
					xhr.send();
				}
				xhr.onabort = function() {
                    if (showindicator||showindicator == undefined) {
                        App.hideIndicator();
                    }
					reject(TIMEOUT_MSG);
					if (xhrTimeout)
						clearTimeout(xhrTimeout);
				};
				xhrTimeout = setTimeout(function() {
                    if (showindicator||showindicator == undefined) {
                        App.hideIndicator();
                    }
					xhr.abort();
				}, timeout);

				xhr.onload = function() {
                    if (showindicator||showindicator == undefined) {
                        App.hideIndicator();
                    }
					if (xhrTimeout)
						clearTimeout(xhrTimeout);
					if (this.status >= 200 && this.status < 300) {
						var res = JSON.parse(this.response);
						if ((res.error_info || "").indexOf("用户未登录") >= 0 || (res.error_info || "").indexOf("用户未登陆") >= 0) {
							exit();
						}
						if (res.error_no == 0 || !res.error_no) {
							resolve(res);
						} else {
							if(res.error_info.indexOf("服务器异常")>=0||res.error_info.length>100){
								reject("服务器异常");
							}else{
								reject(res.error_info);
							}
						}
					} else {
						reject(this.statusText);
					}
				};
				xhr.onerror = function() {
					App.hideIndicator();
					if (xhrTimeout)
						clearTimeout(xhrTimeout);
					reject(this.statusText);
				};
			});
			return promise;
		}
	};

	// Adapter pattern
	return {
		get : function(url, showIndicator) {
			return core.ajax('GET', url, {}, showIndicator);
		},
		get : function(url) {
			return core.ajax('GET', url, {}, true);
		},
		post : function(url, args) {
			return core.ajax('POST', url, args, true);
		},
		post : function(url, args, showIndicator) {
			return core.ajax('POST', url, args, showIndicator);
		},
		put : function(url, args) {
			return core.ajax('PUT', url, args, true);
		}
	};
});
/* eslint-enable */
