define([], function () {

	/**
	 * Bind DOM event to some handler function in controller
	 * @param  {Array} bindings
	 */
    function bindEvents(bindings) {
        if ($$.isArray(bindings) && bindings.length > 0) {
            for (var i = 0; i < bindings.length; i++) {
                if (bindings[i].target) {
                    $$(bindings[i].target).on(bindings[i].event, bindings[i].element, bindings[i].handler);
                } else {
                    $$(bindings[i].element).on(bindings[i].event, bindings[i].handler);
                }
            }
        }
    }
    /**
	 * unBind DOM event to some handler function in controller
     * @param bindings
     */
	function unBindEvents(bindings) {
        if ($$.isArray(bindings) && bindings.length > 0) {
            for (var i =0;i< bindings.length;i++) {
                $$(bindings[i].element).off(bindings[i].event, bindings[i].handler);
            }
        }
    }

	/**
	 * Set the position of buttons, which queried by selector, dynamic
	 * @param {String} selector
	 */
	function setButtonPosition(selector) {
		var pageContent = $$(selector).parents('.page-content');
		$$(selector).removeClass('fixed-bottom');
		if (!isScroll(pageContent[0])) {
			$$(selector).addClass('fixed-bottom');
		}
	}

	/**
	 * Detect whether the element has scrollbar
	 * @param  {HTMLElement}  elem
	 * @return {Boolean}      true: has scrollbar; false: hasn't
	 */
	function isScroll(elem) {
		return elem.scrollHeight > elem.clientHeight;
	}

    /**
     * Converts an object to x-www-form-urlencoded serialization.
     * http://victorblog.com/2012/12/20/make-angularjs-http-service-behave-like-jquery-ajax/
     * @param  {Object} obj
     * @return {String}
     */
    function serialize (obj) {
        var query = '', name, value, fullSubName, subName, subValue, innerObj, i;

        for (name in obj) {
            value = obj[name];

            if (value instanceof Array) {
                for (i = 0; i < value.length; ++i) {
                    subValue = value[i];
                    fullSubName = name + '[' + i + ']';
                    innerObj = {};
                    innerObj[fullSubName] = subValue;
                    //query += param(innerObj) + '&';
                    query += encodeURIComponent(name) + '=' + encodeURIComponent(subValue) + '&';
                }
            } else if (value instanceof Object) {
                for (subName in value) {
                    subValue = value[subName];
                    fullSubName = name + '[' + subName + ']';
                    innerObj = {};
                    innerObj[fullSubName] = subValue;
                    query += serialize(innerObj) + '&';
                }
            } else if (value !== undefined && value !== null)
                query += encodeURIComponent(name) + '=' + encodeURIComponent(value) + '&';
        }

        return query.length ? query.substr(0, query.length - 1) : query;
    }

	/* eslint-enable */

    function templateRender(templateScript, data, target) {
        var template = $$(templateScript).html();
        var complieTemplate = Template7.compile(template);
        var renderTemplate = complieTemplate(data);
        $$(target).append(renderTemplate);
    }
    function templatehtmlRender(templateScript, data, target) {
        var template = $$(templateScript).html();
        var complieTemplate = Template7.compile(template);
        var renderTemplate = complieTemplate(data);
        $$(target).html(renderTemplate);
    }
    return {
        serialize:serialize,
		bindEvents: bindEvents,
		setButtonPosition: setButtonPosition,
        unBindEvents:unBindEvents,
        templateRender: templateRender,
        templatehtmlRender: templatehtmlRender
	};
});
